@@ -8,13 +8,13 @@
 #  modification, are permitted provided that the following conditions are met:
 #
 #   * Redistributions of source code must retain the above copyright notice,
-#     this list of conditions and the following disclaimer.
+#	 this list of conditions and the following disclaimer.
 #   * Redistributions in binary form must reproduce the above copyright notice,
-#     this list of conditions and the following disclaimer in the documentation
-#     and/or other materials provided with the distribution.
+#	 this list of conditions and the following disclaimer in the documentation
+#	 and/or other materials provided with the distribution.
 #   * Neither the name of SWITCH nor the names of its contributors may be
-#     used to endorse or promote products derived from this software without
-#     specific prior written permission.
+#	 used to endorse or promote products derived from this software without
+#	 specific prior written permission.
 #
 #  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 #  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -28,11 +28,11 @@
 #  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 #  POSSIBILITY OF SUCH DAMAGE.
 #
-#  $Author: phaag $
+#  $Author: haag $
 #
-#  $Id: install.pl 24 2007-11-21 09:12:03Z phaag $
+#  $Id: install.pl 67 2010-09-09 05:56:05Z haag $
 #
-#  $LastChangedRevision: 24 $
+#  $LastChangedRevision: 67 $
 
 require 5.6.0;
 
@@ -49,10 +49,11 @@ use Nfsources;
 # from installer-items - helper modules
 use CopyRecursive;
 use File::Copy;
+use File::Path;
 use RRDconvertv1;
 
-my $VERSION = '$Id: install.pl 24 2007-11-21 09:12:03Z phaag $';
-my $nfsen_version = "1.3.2";
+my $VERSION = '$Id: install.pl 67 2010-09-09 05:56:05Z haag $';
+my $nfsen_version = "1.3.5";
 
 my @ProfileTag = ( 
 	"# \n",
@@ -61,9 +62,10 @@ my @ProfileTag = (
 );
 
 my @RequiredDirs = ( 'PREFIX', 'BINDIR', 'LIBEXECDIR', 'CONFDIR', 'DOCDIR', 
-					 'VARDIR', 'PROFILESTATDIR', 'PROFILEDATADIR', 
+					 'VARDIR', 'PIDDIR', 'FILTERDIR', 'PROFILESTATDIR', 'PROFILEDATADIR', 
 					 'BACKEND_PLUGINDIR', 'FRONTEND_PLUGINDIR', 'HTMLDIR' );
 #
+
 sub FindCommand {
 	my $command = shift;
 
@@ -118,7 +120,7 @@ sub CopyDir {
 	my $dest = shift;
 
 	my $DIR;
-	mkdir $dest unless -d $dest;
+	mkpath($dest, 1, 0755) unless -d $dest;
 	opendir($DIR, $orig) || die "can't open dir $orig: $!\n";
 	while ( my $direntry = readdir $DIR ) {
 		next if $direntry eq '.';
@@ -197,6 +199,21 @@ sub VerifyConfig {
 		die "nfdump tools installation error: '$binary' not found in '$NfConf::PREFIX'" unless -f "$NfConf::PREFIX/$binary";
 	}
 
+	my @out = `$NfConf::PREFIX/nfdump -V`;
+	if ( scalar @out != 2 ) {
+		die "Error getting nfdump version";
+	}
+	chomp $out[0];
+	my ($major, $minor) = $out[0] =~ /Version:\s(\d)\.(\d)\s/;
+	if ( defined $major && defined $minor) {
+		if ( $major >= 1 && $minor >= 6 ) {
+			print "Found $out[0]\n";
+		} else {
+			print "out[0]\n";
+			die "Nfdump version not compatible with current NfSen version.\n";
+		}
+	} 
+
 	my $www_gid = getgrnam($NfConf::WWWGROUP) || 
 		die "WWW group '$NfConf::WWWGROUP' not found on this system\n";
 
@@ -209,33 +226,8 @@ sub VerifyConfig {
 		die "Selected subdir hierarchy %NfConf::SUBDIRLAYOUT out of range\n";
 	}
 
-	my %_seen;
-	foreach my $source ( keys %NfConf::sources ) {
-		my $len = length($source);
-		if ( $len < 1 || $len > 19 ) {
-			die "Error Ident: '$source': Source identifiers must be between 1 and 19 chars long!\n";
-		}
-		if ( $source =~ /[^a-z0-9_\-]+/i ) {
-			die "Error Ident: '$source': Source identifiers must contain only characters [a-zA-Z0-9_-] !\n";
-		}
-		my $port = $NfConf::sources{$source}{'port'};
-		if ( $port > 0 && exists $_seen{$port} ) {
-			die "Error Ident: '$source': Port $port already configured for source $_seen{$port}!\n";
-		} else {
-			$_seen{$port} = $source;
-		}
-		# 
-		my $type = $NfConf::sources{$source}{'type'};
-		if ( defined $type ) {
-			$type = lc $type;
-			my $collector = $NfSenRC::CollectorMap{$type};
-
-			if ( !defined $collector ) {
-				die "Source '$source': unknown collector for type '$type'";
-			} elsif ( ! -f "$NfConf::PREFIX/$collector" ) {
-				die "Source '$source': missing collector '$collector' for type '$type'";
-			}
-		}
+	if ( !Nfsources::ValidateSources() ) {
+		die "Fix errors for %source in your config file and retry!\n";
 	}
 
 	if ( !defined $NfConf::low_water ) {
@@ -283,23 +275,23 @@ sub PatchVars {
 sub PatchAllScripts {
 
 	my %GlobList = ( 
-		"bin/*"		=> 	"$NfConf::BINDIR",
-		"libexec/*"	=>	"$NfConf::LIBEXECDIR"
+		"bin/*"		=> 	"$NfConf::INSTPREFIX$NfConf::BINDIR",
+		"libexec/*"	=>	"$NfConf::INSTPREFIX$NfConf::LIBEXECDIR"
 	);
 
 	foreach my $glob_list ( keys %GlobList ) {
 		my $dir 	= $GlobList{$glob_list};
-    	my @Scripts	= glob($glob_list);
+		my @Scripts	= glob($glob_list);
 		print "In directory: $dir ...\n";
-    	foreach my $script (@Scripts) {
-        	$script =~ s#^.+/##;
-        	if ( -f "$dir/$script" ) {
-            	print "Update script: $script\n";
+		foreach my $script (@Scripts) {
+			$script =~ s#^.+/##;
+			if ( -f "$dir/$script" ) {
+				print "Update script: $script\n";
 				PatchVars("$dir/$script");
-        	} else {
-            	print "ERROR: Failed to update script: '$script'. Installation incomplete!\n";
-        	}
-    	}
+			} else {
+				print "ERROR: Failed to update script: '$script'. Installation incomplete!\n";
+			}
+		}
 	}
 
 	print "\n";
@@ -316,8 +308,8 @@ sub RenameFiles {
 			foreach my $profile ( @AllProfiles ) {
 				foreach my $type (  'flows', 'packets', 'traffic' ) {
 					foreach my $periode (  'day', 'week', 'month',  'year' ) {
-						my $oldfile = "$NfConf::PROFILESTATDIR/$profilegroup/$profile/${type}-${periode}.gif";
-						my $newfile = "$NfConf::PROFILESTATDIR/$profilegroup/$profile/${type}-${periode}.png";
+						my $oldfile = "$NfConf::INSTPREFIX$NfConf::PROFILESTATDIR/$profilegroup/$profile/${type}-${periode}.gif";
+						my $newfile = "$NfConf::INSTPREFIX$NfConf::PROFILESTATDIR/$profilegroup/$profile/${type}-${periode}.png";
 						if ( -f $oldfile ) {
 							rename $oldfile, $newfile;
 						}
@@ -334,17 +326,17 @@ sub SetupHTML {
 	my ( $nfsen_uid, $www_gid ) = @_;
 
 	print "Setup php and html files.\n";
-	mkdir "$NfConf::HTMLDIR" unless -d "$NfConf::HTMLDIR";
-	die "Could not create HTMl directory '$NfConf::HTMLDIR': $!\n"  unless -d $NfConf::HTMLDIR;
+	mkpath("$NfConf::INSTPREFIX$NfConf::HTMLDIR", 1, 0755) unless -d "$NfConf::INSTPREFIX$NfConf::HTMLDIR";
+	die "Could not create HTMl directory '$NfConf::INSTPREFIX$NfConf::HTMLDIR': $!\n"  unless -d "$NfConf::INSTPREFIX$NfConf::HTMLDIR";
 
 	$CopyRecursive::CopyLink = 1;
 	$CopyRecursive::MODE 	 = 0644;
 	$CopyRecursive::UID 	 = 0;
 	$CopyRecursive::GID 	 = $www_gid;
-	CopyRecursive::dircopy("html", "$NfConf::HTMLDIR");
-	open CONF, ">$NfConf::HTMLDIR/conf.php" || die "Can't open conf.php for writing: $!\n";
+	CopyRecursive::dircopy("html", "$NfConf::INSTPREFIX$NfConf::HTMLDIR");
+	open CONF, ">$NfConf::INSTPREFIX$NfConf::HTMLDIR/conf.php" || die "Can't open conf.php for writing: $!\n";
 	print CONF "<?php\n";
-	print CONF "/* This file was automatically created by the NfSen install.pl script */\n\n";
+	print CONF "/* This file was automatically created by the NfSen $nfsen_version install.pl script */\n\n";
 	print CONF "\$COMMSOCKET = \"$NfConf::COMMSOCKET\";\n";
 	print CONF "\n\$DEBUG=0;\n";
 	print CONF "\n?>\n";
@@ -362,15 +354,15 @@ sub SetupEnv {
 	umask 0002;
 	print "\nSetup diretories:\n";
 	my @dirs = (
-		"$NfConf::VARDIR",
-		"$NfConf::PIDDIR",
-		"$NfConf::VARDIR/tmp",
-		"$NfConf::VARDIR/filters",
-		"$NfConf::VARDIR/fmt",
-		"$NfConf::PROFILESTATDIR",
-		"$NfConf::PROFILESTATDIR/live",
-		"$NfConf::PROFILEDATADIR",
-		"$NfConf::PROFILEDATADIR/live",
+		"$NfConf::INSTPREFIX$NfConf::VARDIR",
+		"$NfConf::INSTPREFIX$NfConf::VARDIR/tmp",
+		"$NfConf::INSTPREFIX$NfConf::PIDDIR",
+		"$NfConf::INSTPREFIX$NfConf::FILTERDIR",
+		"$NfConf::INSTPREFIX$NfConf::FORMATDIR",
+		"$NfConf::INSTPREFIX$NfConf::PROFILESTATDIR",
+		"$NfConf::INSTPREFIX$NfConf::PROFILESTATDIR/live",
+		"$NfConf::INSTPREFIX$NfConf::PROFILEDATADIR",
+		"$NfConf::INSTPREFIX$NfConf::PROFILEDATADIR/live",
 	);
 
 	print "\nUse UID/GID $nfsen_uid $www_gid\n";
@@ -378,7 +370,7 @@ sub SetupEnv {
 	foreach my $dir ( @dirs ) {
 		if ( ! -d $dir ) {
 			print "Creating: ";
-			mkdir $dir || die "Can't mkdir '$dir': $!\n";
+			mkpath($dir, 1, 0755) || die "Can't mkpath '$dir': $!\n";
 		} else {
 			print "Exists: ";
 		}
@@ -392,7 +384,7 @@ sub SetupEnv {
 		my $dir = "$NfConf::PROFILEDATADIR/live/$ident";
 		if ( ! -d $dir ) {
 			print "Creating: ";
-			mkdir $dir || die "Can't mkdir '$dir': $!\n";
+			mkpath($dir, 1, 0755) || die "Can't mkpath '$dir': $!\n";
 		} else {
 			print "Exists: ";
 		}
@@ -524,9 +516,9 @@ sub UpgradeProfiles {
 				# copy filter files
 
 				if ( $profileinfo{'type'} == 1 ) {
-					$oldfilter    = "$NfConf::PROFILEDATADIR/$profilename/history_filter.txt";
+					$oldfilter	= "$NfConf::PROFILEDATADIR/$profilename/history_filter.txt";
 				} else {
-					$oldfilter    = "$NfConf::PROFILEDATADIR/$profilename/filter.txt";
+					$oldfilter	= "$NfConf::PROFILEDATADIR/$profilename/filter.txt";
 				}
 				my $newfilter = "$NfConf::PROFILESTATDIR/$profilename/$channel-filter.txt";
 
@@ -586,13 +578,12 @@ sub CopyAllFiles {
 	$CopyRecursive::GID 	 = $www_gid;
 	$CopyRecursive::MODE 	 = 0755;
 	unlink "$NfConf::BINDIR/nfsen.rc";
-	CopyRecursive::dircopy("bin", "$NfConf::BINDIR");
-	CopyRecursive::dircopy("libexec", "$NfConf::LIBEXECDIR");
+	CopyRecursive::dircopy("bin", "$NfConf::INSTPREFIX$NfConf::BINDIR");
+	CopyRecursive::dircopy("libexec", "$NfConf::INSTPREFIX$NfConf::LIBEXECDIR");
 	$CopyRecursive::MODE 	 = 0644;
-	CopyRecursive::dircopy("etc", "$NfConf::CONFDIR");
-	CopyRecursive::dircopy("plugins/backend",  "$NfConf::BACKEND_PLUGINDIR");
-	CopyRecursive::dircopy("plugins/frontend", "$NfConf::FRONTEND_PLUGINDIR");
-	CopyRecursive::dircopy("doc", "$NfConf::DOCDIR");
+	CopyRecursive::dircopy("etc", "$NfConf::INSTPREFIX$NfConf::CONFDIR");
+	CopyRecursive::dircopy("plugins/backend",  "$NfConf::INSTPREFIX$NfConf::BACKEND_PLUGINDIR");
+	CopyRecursive::dircopy("plugins/frontend", "$NfConf::INSTPREFIX$NfConf::FRONTEND_PLUGINDIR");
 
 	if ( $ConfigFile eq "$NfConf::CONFDIR/nfsen.conf" ) {
 		print "Keep config file '$ConfigFile'\n";
@@ -696,14 +687,22 @@ if ( !defined $NfConf::RRDoffset ) {
 	die "$Log::ERROR\n";
 }
 
-
 # 
 print "Setup NfSen:\n";
 print "Version: $nfsen_version: $VERSION\n\n";
 
 # Get Perl
-# Put this into a NfCOnf Variablee, so we can use the standard Patch Procedure
+# Put this into a NfConf variable, so we can use the standard Patch Procedure
 $NfConf::PERL = GetPerl();
+$NfConf::INSTPREFIX = $ENV{'INSTPREFIX'};
+if ( defined $NfConf::INSTPREFIX ) {
+	if ( ! $NfConf::INSTPREFIX =~ /\/$/ ) {
+		$NfConf::INSTPREFIX = $NfConf::INSTPREFIX . '/';	# make sure path ends with a '/' 
+	}
+	print "Install prefixdir: '$NfConf::INSTPREFIX'\n";
+} else {
+	$NfConf::INSTPREFIX = '';
+}
 
 
 my ($nfsen_uid, $www_gid ) = VerifyConfig();
@@ -722,36 +721,40 @@ if ( $need_rrdlayout_upgrade ) {
 }
 
 my $pid_name = "nfsend.pid";
-if ( -f "$NfConf::VARDIR/run/nfsen-run.pid" ) {
-	# from NfSen <= 1.2
-	$pid_name = "nfsen-run.pid";
-}
 
-if ( -f "$NfConf::VARDIR/run/$pid_name" ) {
-	open PID, "$NfConf::VARDIR/run/$pid_name" || die "Can't open pid file: $!\n";
-	my $pid = <PID>;
-	chomp($pid);
-	close PID;
-	die "Can't extract PID out of '$NfConf::VARDIR/run/$pid_name'. Stop upgrade" if !defined $pid;
-	if ( kill(0, $pid) == 1 ) {
-		print "Stop nfsend while upgrading .";
-		kill 15, $pid;
-		my $cnt = 0;
-		while ( -f "$NfConf::VARDIR/run/$pid_name" && $cnt < 300 ) {
-			print ".";
-			$cnt++;
-			sleep(1);
-		}
-		if ( -f "$NfConf::VARDIR/run/$pid_name" ) {
-			print "\nnfsend doesn't want to die! It's not save to upgrade NfSen!\n";
-			exit;
-		} else {
-			print "done.\n";
-			$nfsen_run = 1;
-		}
+if ( -f "$NfConf::PIDDIR/$pid_name" ) {
+	my ($major, $minor, $release) = split /\./, $$$hints{'version'};
+	if ( $release < 5 ) {
+		# for older releases than 1.3.5 upgrade we to stop nfsen completely
+		print "Stop NfSen completely while upgrading to version $nfsen_version\n";
+		$nfsen_run = 2;	# a complete restart needed
+		system("$NfConf::BINDIR/nfsen stop");
 	} else {
-		print "nfsend pid file exists, but no process is running.\n";
-		unlink "$NfConf::VARDIR/run/$pid_name";
+		open PID, "$NfConf::PIDDIR/$pid_name" || die "Can't open pid file: $!\n";
+		my $pid = <PID>;
+		chomp($pid);
+		close PID;
+		die "Can't extract PID out of '$NfConf::PIDDIR/$pid_name'. Stop upgrade" if !defined $pid;
+		if ( kill(0, $pid) == 1 ) {
+			print "Stop nfsend while upgrading .";
+			kill 15, $pid;
+			my $cnt = 0;
+			while ( -f "$NfConf::PIDDIR/$pid_name" && $cnt < 300 ) {
+				print ".";
+				$cnt++;
+				sleep(1);
+			}
+			if ( -f "$NfConf::PIDDIR/$pid_name" ) {
+				print "\nnfsend doesn't want to die! It's not save to upgrade NfSen!\n";
+				exit;
+			} else {
+				print "done.\n";
+				$nfsen_run = 1;
+			}
+		} else {
+			print "nfsend pid file exists, but no process is running.\n";
+			unlink "$NfConf::PIDDIR/$pid_name";
+		}
 	}
 }
 
@@ -780,22 +783,36 @@ if ( $need_rrdlayout_upgrade ) {
 	}
 }
 
+if ( !exists $$$hints{'sources'} ) {
+	foreach my $source ( sort keys %NfConf::sources ) {
+		my $port = $NfConf::sources{$source}{'port'};
+		$$$hints{'sources'}{$source} = $port;
+	}
+}
 
 Nfsources::Reconfig();
+delete $$$hints{'sources'};
+foreach my $source ( sort keys %NfConf::sources ) {
+	$$$hints{'sources'}{$source} = $NfConf::sources{$source}{'port'};
+}
+
 
 $$$hints{'version'} 		= $nfsen_version;
 $$$hints{'installed'} 		= time();
 NfSen::StoreHints();
-chown $nfsen_uid, $www_gid, "$NfConf::PROFILESTATDIR/hints" || die "Can't chown hints db: $!\n";
+chown $nfsen_uid, $www_gid, "$NfConf::INSTPREFIX$NfConf::PROFILESTATDIR/hints" || die "Can't chown hints db: $!\n";
 print "Setup done.\n\n";
 
-if ( $nfsen_run ) {
+if ( $nfsen_run == 2 ) {
+	print "Restart NfSen\n";
+	system("$NfConf::BINDIR/nfsen start");
+} elsif ( $nfsen_run == 1 ) {
 	print "Restart nfsend\n";
 	system("$NfConf::BINDIR/nfsend");
-}
+} 
 
 print "* You may want to subscribe to the nfsen-discuss mailing list:\n";
 print "* http://lists.sourceforge.net/lists/listinfo/nfsen-discuss\n";
-print "* Please send bug reports back to me: peter.haag\@switch.ch\n";
+print "* Please send bug reports back to me: phaag\@sourceforge.net\n";
 exit 0;
 
